import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.byteman.contrib.bmunit.BMRule;
import org.jboss.byteman.contrib.bmunit.BMRules;
import org.jboss.byteman.contrib.bmunit.BMUnitRunner;
import org.junit.runner.RunWith;

/**
 * @author jmartisk
 * @since 5/14/13
 */
@RunWith(BMUnitRunner.class)
public class ClientTest {

    @org.junit.Test
    @BMRules(rules = {
            @BMRule(name="initialize countdown",
                    targetClass = "EJBInvocationHandler",
                    targetMethod = "<init>",
                    action = "createCountDown(\"cd\", 3); "
                            + "System.out.println(\"initializing countdown..\");"),
            @BMRule(name = "throw RuntimeException",
                    targetClass = "MethodInvocationMessageWriter",
                    targetMethod = "writeMessage",
                    condition = "countDown(\"cd\")",
                    action = "createCountDown(\"cd\", 5);"
                            + "System.out.println(\"[byteman] INJECTING IOException to simulate networking problem\");"
                            + "throw new java.io.IOException(\"BYTEMAN-INJECTED EXCEPTION\"); "
            )
    })
    public void doTest() throws NamingException {

        Properties p = new Properties();
        p.put("org.jboss.ejb.client.scoped.context", true);
        p.put(Context.URL_PKG_PREFIXES, "org.jboss.ejb.client.naming");
        p.put("remote.connectionprovider.create.options.org.xnio.Options.SSL_ENABLED", "false");
        p.put("remote.connections", "one");
        p.put("remote.connection.one.port", "4447");
        p.put("remote.connection.one.host", "localhost");

        Context ctx = new InitialContext(p);
        try {
            BeanRemote bean = (BeanRemote)ctx.lookup("ejb:/server-side/Bean!BeanRemote");
            for (int i = 0; i < 50; i++) {
                try {
                    System.out.print("invoking... ");
                    String ret = bean.doStuff();
                    System.out.println(ret);
                    Thread.sleep(500);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        } finally {
            ctx.close();
        }

    }
}
